@@ -16,26 +16,37 @@
 #
 
 import logging
-from skywalking.loggings import logger
 import traceback
-from queue import Queue, Empty, Full
+from queue import Queue, Empty
 from time import time
 
 import grpc
+from skywalking.protocol.common.Common_pb2 import KeyStringValuePair
+from skywalking.protocol.language_agent.Tracing_pb2 import SegmentObject, SpanObject, Log, SegmentReference
+from skywalking.protocol.logging.Logging_pb2 import LogData
+from skywalking.protocol.profile.Profile_pb2 import ThreadSnapshot, ThreadStack
 
 from skywalking import config
 from skywalking.agent import Protocol
 from skywalking.agent.protocol.interceptors import header_adder_interceptor
-from skywalking.client.grpc import GrpcServiceManagementClient, GrpcTraceSegmentReportService
-from skywalking.protocol.common.Common_pb2 import KeyStringValuePair
-from skywalking.protocol.language_agent.Tracing_pb2 import SegmentObject, SpanObject, Log, SegmentReference
+from skywalking.client.grpc import GrpcServiceManagementClient, GrpcTraceSegmentReportService, \
+    GrpcProfileTaskChannelService, GrpcLogDataReportService
+from skywalking.loggings import logger
 from skywalking.trace.segment import Segment
+from skywalking.profile.snapshot import TracingThreadSnapshot
+from skywalking.profile.profile_task import ProfileTask
 
 
 class GrpcProtocol(Protocol):
     def __init__(self):
+        self.properties_sent = False
         self.state = None
-        self.channel = grpc.insecure_channel(config.collector_address)
+
+        if config.force_tls:
+            self.channel = grpc.secure_channel(config.collector_address, grpc.ssl_channel_credentials())
+        else:
+            self.channel = grpc.insecure_channel(config.collector_address)
+
         if config.authentication:
             self.channel = grpc.intercept_channel(
                 self.channel, header_adder_interceptor('authentication', config.authentication)
@@ -44,24 +55,31 @@ class GrpcProtocol(Protocol):
         self.channel.subscribe(self._cb, try_to_connect=True)
         self.service_management = GrpcServiceManagementClient(self.channel)
         self.traces_reporter = GrpcTraceSegmentReportService(self.channel)
+        self.profile_channel = GrpcProfileTaskChannelService(self.channel)
+        self.log_reporter = GrpcLogDataReportService(self.channel)
 
     def _cb(self, state):
         logger.debug('grpc channel connectivity changed, [%s -> %s]', self.state, state)
         self.state = state
-        if self.connected():
-            try:
-                self.service_management.send_instance_props()
-            except grpc.RpcError:
-                self.on_error()
+
+    def query_profile_commands(self):
+        logger.debug("query profile commands")
+        self.profile_channel.do_query()
+
+    def notify_profile_task_finish(self, task: ProfileTask):
+        self.profile_channel.finish(task)
 
     def heartbeat(self):
         try:
+            if not self.properties_sent:
+                self.service_management.send_instance_props()
+                self.properties_sent = True
+
             self.service_management.send_heart_beat()
+
         except grpc.RpcError:
             self.on_error()
-
-    def connected(self):
-        return self.state == grpc.ChannelConnectivity.READY
+            raise
 
     def on_error(self):
         traceback.print_exc() if logger.isEnabledFor(logging.DEBUG) else None
@@ -69,19 +87,26 @@ class GrpcProtocol(Protocol):
         self.channel.subscribe(self._cb, try_to_connect=True)
 
     def report(self, queue: Queue, block: bool = True):
-        start = time()
-        segment = None
+        start = None
 
         def generator():
-            nonlocal segment
+            nonlocal start
 
             while True:
                 try:
-                    timeout = max(0, config.QUEUE_TIMEOUT - int(time() - start))  # type: int
+                    timeout = config.QUEUE_TIMEOUT  # type: int
+                    if not start:  # make sure first time through queue is always checked
+                        start = time()
+                    else:
+                        timeout -= int(time() - start)
+                        if timeout <= 0:  # this is to make sure we exit eventually instead of being fed continuously
+                            return
                     segment = queue.get(block=block, timeout=timeout)  # type: Segment
                 except Empty:
                     return
 
+                queue.task_done()
+
                 logger.debug('reporting segment %s', segment)
 
                 s = SegmentObject(
@@ -105,9 +130,9 @@ class GrpcProtocol(Protocol):
                             data=[KeyStringValuePair(key=item.key, value=item.val) for item in log.items],
                         ) for log in span.logs],
                         tags=[KeyStringValuePair(
-                            key=str(tag.key),
+                            key=tag.key,
                             value=str(tag.val),
-                        ) for tag in span.tags],
+                        ) for tag in span.iter_tags()],
                         refs=[SegmentReference(
                             refType=0 if ref.ref_type == "CrossProcess" else 1,
                             traceId=ref.trace_id,
@@ -123,16 +148,76 @@ class GrpcProtocol(Protocol):
 
                 yield s
 
-                queue.task_done()
-
         try:
             self.traces_reporter.report(generator())
-
         except grpc.RpcError:
             self.on_error()
+            raise  # reraise so that incremental reconnect wait can process
 
-            if segment:
+    def report_log(self, queue: Queue, block: bool = True):
+        start = None
+
+        def generator():
+            nonlocal start
+
+            while True:
                 try:
-                    queue.put(segment, block=False)
-                except Full:
-                    pass
+                    timeout = config.QUEUE_TIMEOUT  # type: int
+                    if not start:  # make sure first time through queue is always checked
+                        start = time()
+                    else:
+                        timeout -= int(time() - start)
+                        if timeout <= 0:  # this is to make sure we exit eventually instead of being fed continuously
+                            return
+                    log_data = queue.get(block=block, timeout=timeout)  # type: LogData
+                except Empty:
+                    return
+
+                queue.task_done()
+
+                logger.debug('Reporting Log')
+
+                yield log_data
+
+        try:
+            self.log_reporter.report(generator())
+        except grpc.RpcError:
+            self.on_error()
+            raise
+
+    def send_snapshot(self, queue: Queue, block: bool = True):
+        start = None
+
+        def generator():
+            nonlocal start
+
+            while True:
+                try:
+                    timeout = config.QUEUE_TIMEOUT  # type: int
+                    if not start:  # make sure first time through queue is always checked
+                        start = time()
+                    else:
+                        timeout -= int(time() - start)
+                        if timeout <= 0:  # this is to make sure we exit eventually instead of being fed continuously
+                            return
+                    snapshot = queue.get(block=block, timeout=timeout)  # type: TracingThreadSnapshot
+                except Empty:
+                    return
+
+                queue.task_done()
+
+                transform_snapshot = ThreadSnapshot(
+                    taskId=str(snapshot.task_id),
+                    traceSegmentId=str(snapshot.trace_segment_id),
+                    time=int(snapshot.time),
+                    sequence=int(snapshot.sequence),
+                    stack=ThreadStack(codeSignatures=snapshot.stack_list)
+                )
+
+                yield transform_snapshot
+
+        try:
+            self.profile_channel.send(generator())
+        except grpc.RpcError:
+            self.on_error()
+            raise
