@@ -27,3 +27,16 @@ class ServiceManagementClient(object):
 class TraceSegmentReportService(object):
     def report(self, generator):
         raise NotImplementedError()
+
+
+class LogDataReportService(object):
+    def report(self, generator):
+        raise NotImplementedError()
+
+
+class ProfileTaskChannelService(object):
+    def do_query(self):
+        raise NotImplementedError()
+
+    def send(self, generator):
+        raise NotImplementedError()
