@@ -17,11 +17,16 @@
 
 from inspect import iscoroutinefunction, isawaitable
 
-from skywalking import Layer, Component
-from skywalking.trace import tags
+from skywalking import Layer, Component, config
 from skywalking.trace.carrier import Carrier
-from skywalking.trace.context import get_context
-from skywalking.trace.tags import Tag
+from skywalking.trace.context import get_context, NoopContext
+from skywalking.trace.span import NoopSpan
+from skywalking.trace.tags import TagHttpMethod, TagHttpURL, TagHttpStatusCode
+
+version_rule = {
+    "name": "tornado",
+    "rules": [">=5.0"]
+}
 
 
 def install():
@@ -50,23 +55,27 @@ def _gen_sw_get_response_func(old_execute):
         # In that case our method should be a coroutine function too
         async def _sw_get_response(self, *args, **kwargs):
             request = self.request
-            context = get_context()
             carrier = Carrier()
+            method = request.method
+
             for item in carrier:
                 if item.key.capitalize() in request.headers:
                     item.val = request.headers[item.key.capitalize()]
-            with context.new_entry_span(op=request.path, carrier=carrier) as span:
+
+            span = NoopSpan(NoopContext()) if config.ignore_http_method_check(method) \
+                else get_context().new_entry_span(op=request.path, carrier=carrier)
+
+            with span:
                 span.layer = Layer.Http
                 span.component = Component.Tornado
                 peer = request.connection.stream.socket.getpeername()
                 span.peer = '{0}:{1}'.format(*peer)
-                span.tag(Tag(key=tags.HttpMethod, val=request.method))
-                span.tag(
-                    Tag(key=tags.HttpUrl, val='{}://{}{}'.format(request.protocol, request.host, request.path)))
+                span.tag(TagHttpMethod(method))
+                span.tag(TagHttpURL('{}://{}{}'.format(request.protocol, request.host, request.path)))
                 result = old_execute(self, *args, **kwargs)
                 if isawaitable(result):
                     result = await result
-                span.tag(Tag(key=tags.HttpStatus, val=self._status_code, overridable=True))
+                span.tag(TagHttpStatusCode(self._status_code))
                 if self._status_code >= 400:
                     span.error_occurred = True
             return result
@@ -74,21 +83,25 @@ def _gen_sw_get_response_func(old_execute):
         @coroutine
         def _sw_get_response(self, *args, **kwargs):
             request = self.request
-            context = get_context()
             carrier = Carrier()
+            method = request.method
+
             for item in carrier:
                 if item.key.capitalize() in request.headers:
                     item.val = request.headers[item.key.capitalize()]
-            with context.new_entry_span(op=request.path, carrier=carrier) as span:
+
+            span = NoopSpan(NoopContext()) if config.ignore_http_method_check(method) \
+                else get_context().new_entry_span(op=request.path, carrier=carrier)
+
+            with span:
                 span.layer = Layer.Http
                 span.component = Component.Tornado
                 peer = request.connection.stream.socket.getpeername()
                 span.peer = '{0}:{1}'.format(*peer)
-                span.tag(Tag(key=tags.HttpMethod, val=request.method))
-                span.tag(
-                    Tag(key=tags.HttpUrl, val='{}://{}{}'.format(request.protocol, request.host, request.path)))
+                span.tag(TagHttpMethod(method))
+                span.tag(TagHttpURL('{}://{}{}'.format(request.protocol, request.host, request.path)))
                 result = yield from old_execute(self, *args, **kwargs)
-                span.tag(Tag(key=tags.HttpStatus, val=self._status_code, overridable=True))
+                span.tag(TagHttpStatusCode(self._status_code))
                 if self._status_code >= 400:
                     span.error_occurred = True
             return result
